/*************************************************************************

s1m.h - constants for s1m file format

This file is a part of sscsss -- Simple Single Cycle Sample Set Synthesizer.

Copyright (C) 2002 David Baker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#ifndef _SSCSSS_S1M_H
#define _SSCSSS_S1M_H

#include "sscsss.h"

/* Some EX S1M File related constants */

/*
   notes - multibyte numbers are bigendian
	 - at end of sample data (or header if no sample data),
	   fill with 0's to 128 byte boundary
	 - wave records are followed by layer records for that wave,
	   i.e. they are mixed together
	 - contrary to the ex manual, I believe the fraction is subtracted
	   from the loop length (based on fft experiments)
	 - the ex appears to use two samples after the loop end, again based
	   on fft experiments (? quadratic interpolation past the loop end
	   for fractional sample point numbers ?)
*/


#define INITSIZE     69888	/* size of initialized s1m file		    */

#define S1M_SR       44100	/* sample rate (hz)			    */
#define S1M_XS           2	/* extra sample points			    */
#define S1M_WPV          4	/* waves per voice			    */
#define S1M_WPF       1024	/* waves per file			    */
#define S1M_SPW        128	/* samples (or layers) per wave		    */
#define S1M_SPF       1024	/* samples per file			    */

/* start addresses of major sections */

#define WAVEOFFSET 0x000086	/* start of wave offsets		    */
/* 1024 4 byte integers, offsets into WAVEPARAMS,			    */
/* wave number - 1 is the position in this array			    */
#define WAVEPARAMS 0x001086	/* start of wave parameters		    */
/* 1024 12 byte wave records and 1024 18 byte layer records		    */
#define SHDROFFSET 0x008886	/* start of sample records		    */
/* 1024 30 records, position is this array is sample number - 1,	    */
/* SORDER gives position in sample data					    */
#define SSIZOFFSET 0x010086	/* start of sample sizes (bytes)	    */
#define SDATOFFSET 0x011086	/* start of sample data			    */

/* odds and ends */

#define SMPLSIZE   0x00007f	/* rough size of sample data ?		    */
#define NEXTWAVE   0x000082	/* offset of next wave record		    */
#define NUMLAYERS  0x000084	/* number of wave layers		    */

/* offsets of wave parameters, not complete */

#define WNAME		 0x00	/* name				 -- 8 bytes */
#define WLAYERS		 0x09	/* number of layers		 -- 1 byte  */
#define WCATEGORY	 0x0b	/* category code		 -- 1 byte  */

/* offsets of wave layer parameters, not complete */

#define LSMPLNO		 0x00	/* sample number   		 -- 2 bytes */
#define LLEVEL		 0x02	/* level			 -- 1 byte  */
/* 0 in file is 127 on ex display, 127 in file is 0 on ex display ?	    */
#define LMYS1		 0x03	/* ?				 -- 1 byte  */
#define LLOWNOTE	 0x04	/* low note (MIDI)		 -- 1 byte  */
#define LHIGHNOTE	 0x05	/* high note			 -- 1 byte  */
#define LLOWVEL		 0x06	/* low velocity (1-127)		 -- 1 byte  */
#define LHIGHVEL	 0x07	/* high velocity		 -- 1 byte  */
#define LPAN		 0x08	/* pan, also freq mode		 -- 1 byte  */
#define LCOARSETUNE	 0x09	/* coarse tune			 -- 1 byte  */
#define LFINETUNE	 0x0a	/* fine tune			 -- 1 byte  */
#define LSMPLPLAY	 0x0b	/* sample play code		 -- 1 byte  */
#define LMYS2		 0x0c	/* ?				 -- 2 bytes */
#define LSTARTOFF	 0x0e	/* start offset			 -- 2 bytes */
#define LLENGTHOFF	 0x10	/* length offset		 -- 2 bytes */

/* offsets of sample parameters, not complete */

#define SNAME            0x00	/* sample name			 -- 8 bytes */
#define SCAT		 0x08   /* sample category		 -- 1 byte  */
#define S09		 0x09	/* ? put 0			 -- 1 byte  */
#define S0A		 0x0a	/* ? put 0xfc			 -- 1 byte  */
#define SMYS1            0x0b	/* ?				 -- 2 bytes */
/* the above appears to be related to cumulative sample length,		    */
/* but the ex recalculates it, so I set it to zero			    */
#define SFRACT           0x0d	/* fraction			 -- 1 byte  */
#define STOPMSTART       0x0e	/* top - start			 -- 3 bytes */
#define STUNE            0x11	/* tune				 -- 1 bytes */
#define SLEN             0x12	/* loop length			 -- 3 bytes */
#define SCENTER          0x15	/* center			 -- 1 byte  */
#define SSTART           0x16	/* start			 -- 3 bytes */
#define SENDDATAMENDLOOP 0x19	/* end data - end loop		 -- 3 bytes */
#define SORDER           0x1c	/* order in data		 -- 2 bytes */

#endif

/* end of s1m.h */

